local ArcanumGuildHall = _G['ArcanumGuildHall']

local LAM2 = LibAddonMenu2
local LMP = LibMediaProvider
local panelOpen = false

local outlineStyles = { 'none', 'outline', 'thin-outline', 'thick-outline', 'shadow', 'soft-shadow-thin', 'soft-shadow-thick' }

local leaveOption = { }
local leaveOptionLookup = { }
for i = 0, 2 do
    table.insert(leaveOption, i)
    table.insert(leaveOptionLookup, ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOGUILDLEAVE_CHOICE" .. i))
end

local endeavorAlertLocation = { }
local endeavorAlertLocationLookup = { }
for i = 0, 1 do
    table.insert(endeavorAlertLocation, i)
    table.insert(endeavorAlertLocationLookup, ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ENDEAVOR_CHOICE" .. i))
end

local showRankIcon = { }
local showRankIconLabels = { }
for i = 0, 2 do
    table.insert(showRankIcon, i)
    table.insert(showRankIconLabels, ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_SHOWRANKICON_CHOICE" .. i))
end

local repairOption = { }
local repairOptionLookup = { }
for i = 0, 2 do
    table.insert(repairOption, i)
    table.insert(repairOptionLookup, ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_STORES_CHOICE" .. i))
end

local rechargeOption = { }
local rechargeOptionLookup = { }
for i = 0, 2 do
    table.insert(rechargeOption, i)
    table.insert(rechargeOptionLookup, ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_RECHARGE_CHOICE" .. i))
end

local selectReminderDay = { os.date("%A", 1633932000), os.date("%A", 1634018400), os.date("%A", 1634104800), os.date("%A", 1634191200), os.date("%A", 1634277600), os.date("%A", 1634364000), os.date("%A", 1634450400) }

function ArcanumGuildHall:InitializeMenu()
    local panelData = {
        type = "panel",
        name = self.addonName,
        displayName = self.displayName,
        author = self.author,
        version = self.version,
        slashCommand = self.slashCommand,
        website = self.website,
        registerForRefresh = true,
        registerForDefaults = true,
    }

    local optionsTable = {
        {
            type = "header",
            name = GetString(SI_KEYBINDINGS_GENERIC_CATEGORY_NAME),
            width = "full",
        },
        {
            type = "checkbox",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CHAT_ICON"),
            getFunc = function()
                return self.db.showChatIcon
            end,
            setFunc = function(show)
                self.db.showChatIcon = show
                self:ShowChatIcon(show)
            end,
        },
        {
            type = "checkbox",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_MONOCHROME_ICON"),
            getFunc = function()
                return self.db.monochromeIcon
            end,
            setFunc = function(monochrome)
                self.db.monochromeIcon = monochrome
                self:SetChatIconTexture(monochrome)
            end,
            disabled = function()
                return not self.db.showChatIcon
            end
        },
        {
            type = "dropdown",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOGUILDLEAVE"),
            tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_NOGUILDLEAVE"),
            choices = leaveOptionLookup,
            choicesValues = leaveOption,
            getFunc = function()
                return self.db.noGuildLeave
            end,
            setFunc = function(value)
                self.db.noGuildLeave = value
                self:CheckNoGuildLeave()
            end,
            default = self.db.noGuildLeave,
        },
        {
            type = "checkbox",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_SHOW_CHANGELOG"),
            getFunc = function()
                return self.db.showChangelog
            end,
            setFunc = function(value)
                self.db.showChangelog = value
            end,
            default = self.db.showChangelog,
        },
        {
            type = "submenu",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_HEADER_NAMES"),
            tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_MEMBER"),
            width = "full",
            controls = {
                {
                    type = "description",
                    title = nil,
                    text = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_HEADER_NAMES_DESCRIPTION"),
                    width = "full",
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_COLORIZE_GUILD_NAMES"),
                    getFunc = function()
                        return self.db.colorizeNames
                    end,
                    setFunc = function(colorize)
                        self.db.colorizeNames = colorize
                    end,
                },
            },
        },
        {
            type = "submenu",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_HEADER_NOTIFICATION"),
            tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_NOTIFICATIONS"),
            width = "full",
            controls = {
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_ACTIVE"),
                    getFunc = function()
                        return self.db.enableReminder
                    end,
                    setFunc = function(use)
                        self.db.enableReminder = use
                        self.db.showReminder = use
                    end,
                },
                {
                    type = "dropdown",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_DAYS"),
                    choices = selectReminderDay,
                    getFunc = function()
                        return self.db.selectedReminderDay
                    end,
                    setFunc = function(value)
                        self.db.selectedReminderDay = value
                        self.db.showReminder = true
                        self:ShowNotification()
                    end,
                    default = self.defaults.selectedReminderDay,
                    width = "full",
                    disabled = function()
                        return not self.db.enableReminder
                    end
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_CHANGE_ACTIVE"),
                    getFunc = function()
                        return self.db.enableNotChange
                    end,
                    setFunc = function(use)
                        self.db.enableNotChange = use
                    end,
                    default = self.db.enableNotChange,
                    disabled = function()
                        return not self.db.enableReminder
                    end
                },
                {
                    type = "editbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_CHANGE_HEADER"),
                    tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_HEADER_TOOLTIP"),
                    getFunc = function()
                        return self.db.defaultNotTitle
                    end,
                    setFunc = function(text)
                        self.db.defaultNotTitle = text
                    end,
                    isMultiline = false,
                    isExtraWide = false,
                    width = "full",
                    default = self.db.defaultNotTitle,
                    warning = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_WARN_HEADER"),
                    disabled = function()
                        return not self.db.enableNotChange or not self.db.enableReminder
                    end
                },
                {
                    type = "button",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_RESET"),
                    func = function()
                        self.db.defaultNotTitle = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_TITLE")
                    end,
                    width = "full",
                    disabled = function()
                        return not self.db.enableNotChange or not self.db.enableReminder
                    end
                },
                {
                    type = "editbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_CHANGE_MESSAGE"),
                    tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_MESSAGE_TOOLTIP"),
                    getFunc = function()
                        return self.db.defaultNotMessage
                    end,
                    setFunc = function(text)
                        self.db.defaultNotMessage = text
                    end,
                    isMultiline = true,
                    isExtraWide = false,
                    width = "full",
                    default = self.db.defaultNotMessage,
                    warning = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_WARN_MESSAGE"),
                    disabled = function()
                        return not self.db.enableNotChange or not self.db.enableReminder
                    end
                },
                {
                    type = "button",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_RESET"),
                    func = function()
                        self.db.defaultNotMessage = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_NOTIFICATION_TEXT")
                    end,
                    width = "full",
                    disabled = function()
                        return not self.db.enableNotChange or not self.db.enableReminder
                    end
                },
            },
        },
        {
            type = "submenu",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ENDEAVOR_TITLE"),
            tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_ENDEAVORS"),
            width = "full",
            controls = {
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ENDEAVOR_ENABLE"),
                    getFunc = function()
                        return self.db.enableEndeavorAlert
                    end,
                    setFunc = function(enable)
                        self.db.enableEndeavorAlert = enable
                    end,
                    default = self.db.enableEndeavorAlert,
                },
                {
                    type = "dropdown",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ENDEAVOR_LOCATION"),
                    choices = endeavorAlertLocationLookup,
                    choicesValues = endeavorAlertLocation,
                    getFunc = function()
                        return self.db.locationEndeavorAlert
                    end,
                    setFunc = function(value)
                        self.db.locationEndeavorAlert = value
                    end,
                    width = "full",
                    default = self.db.locationEndeavorAlert,
                    disabled = function()
                        return not self.db.enableEndeavorAlert
                    end
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ENDEAVOR_ICON"),
                    getFunc = function()
                        return self.db.showEndeavorIcon
                    end,
                    setFunc = function(enable)
                        self.db.showEndeavorIcon = enable
                    end,
                    default = self.db.showEndeavorIcon,
                    disabled = function()
                        return not self.db.enableEndeavorAlert
                    end
                },
                {
                    type = "colorpicker",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ENDEAVOR_COLOR_DAILY"),
                    getFunc = function()
                        return self:ConvHexToRGB(self.db.endeavorColorDaily)
                    end,
                    setFunc = function(r, g, b)
                        self.db.endeavorColorDaily = self:ConvRGBToHex(r, g, b)
                    end,
                    disabled = function()
                        return not self.db.enableEndeavorAlert
                    end
                },
                {
                    type = "colorpicker",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ENDEAVOR_COLOR_WEEKLY"),
                    getFunc = function()
                        return self:ConvHexToRGB(self.db.endeavorColorWeekly)
                    end,
                    setFunc = function(r, g, b)
                        self.db.endeavorColorWeekly = self:ConvRGBToHex(r, g, b)
                    end,
                    disabled = function()
                        return not self.db.enableEndeavorAlert
                    end
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ENDEAVOR_REWARD"),
                    getFunc = function()
                        return self.db.showEndeavorReward
                    end,
                    setFunc = function(enable)
                        self.db.showEndeavorReward = enable
                    end,
                    default = self.db.showEndeavorReward,
                    disabled = function()
                        return not self.db.enableEndeavorAlert
                    end
                },
            },
        },
        {
            type = "submenu",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_TITLE"),
            tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_CLOCK"),
            width = "full",
            controls = {
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_SHOW"),
                    getFunc = function()
                        return self.db.showClock
                    end,
                    setFunc = function(value)
                        self.db.showClock = value
                        self:InitClock()
                        if value == true then
                            ArcanumGuildHall:UpdateAnchors(true)
                        end
                    end,
                    default = self.db.showClock,
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_DST"),
                    getFunc = function()
                        return self.db.clockDst
                    end,
                    setFunc = function(value)
                        self.db.clockDst = value
                        self:UpdateTime()
                    end,
                    default = self.db.showClock,
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_COMBAT"),
                    getFunc = function()
                        return self.db.isInCombatClock
                    end,
                    setFunc = function(value)
                        self.db.isInCombatClock = value
                    end,
                    default = self.db.isInCombatClock,
                    disabled = function()
                        return not self.db.showClock
                    end
                },
                {
                    type = "colorpicker",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_COLOR"),
                    getFunc = function()
                        return self:ConvHexToRGB(self.db.clockFontColor)
                    end,
                    setFunc = function(r, g, b)
                        self.db.clockFontColor = self:ConvRGBToHex(r, g, b)
                        self:UpdateClockStyle()
                    end,
                    disabled = function()
                        return not self.db.showClock
                    end
                },
                {
                    type = "dropdown",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_FONT"),
                    choices = LMP:List("font"),
                    getFunc = function()
                        return self.db.clockTextFont
                    end,
                    setFunc = function(value)
                        self.db.clockTextFont = value
                        self:UpdateClockStyle()
                    end,
                    disabled = function()
                        return not self.db.showClock
                    end
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_SHOW_BG"),
                    getFunc = function()
                        return self.db.showClockBG
                    end,
                    setFunc = function(value)
                        self.db.showClockBG = value
                        self:UpdateClockStyle()
                    end,
                    default = self.db.showClockBG,
                },
                {
                    type = "dropdown",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_BG"),
                    choices = LMP:List("background"),
                    getFunc = function()
                        return self.db.clockBackground
                    end,
                    setFunc = function(value)
                        self.db.clockBackground = value
                        self:UpdateClockStyle()
                    end,
                    disabled = function()
                        return not self.db.showClock
                    end
                },
                {
                    type = "colorpicker",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_BGCOLOR"),
                    tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_BGCOLOR"),
                    getFunc = function()
                        return self:ConvHexToRGB(self.db.clockBackgroundColor)
                    end,
                    setFunc = function(r, g, b)
                        self.db.clockBackgroundColor = self:ConvRGBToHex(r, g, b)
                        self:UpdateClockStyle()
                    end,
                    disabled = function()
                        return not self.db.showClock
                    end
                },
                {
                    type = "slider",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_BGTRANSPARENCY"),
                    min = 0,
                    max = 100,
                    getFunc = function()
                        return self.db.clockBackgroundAlpha
                    end,
                    setFunc = function(value)
                        self.db.clockBackgroundAlpha = value
                        self:UpdateClockStyle()
                    end,
                    disabled = function()
                        return not self.db.showClock
                    end
                },
                {
                    type = "slider",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_FONTSIZE"),
                    min = 8,
                    max = 32,
                    getFunc = function()
                        return self.db.clockFontSize
                    end,
                    setFunc = function(value)
                        self.db.clockFontSize = value
                        self:UpdateClockStyle()
                    end,
                    disabled = function()
                        return not self.db.showClock
                    end
                },
                {
                    type = "dropdown",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_OUTLINE"),
                    choices = outlineStyles,
                    getFunc = function()
                        return self.db.clockFontOutline
                    end,
                    setFunc = function(value)
                        self.db.clockFontOutline = value
                        self:UpdateClockStyle()
                    end,
                    disabled = function()
                        return not self.db.showClock
                    end
                },
                {
                    type = "header",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_HEADER_INGAMETIME"),
                    width = "half"
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_SHOW_INGAMETIME"),
                    getFunc = function()
                        return self.db.showInGameTime
                    end,
                    setFunc = function(value)
                        self.db.showInGameTime = value
                        self:GetInGameTime()
                        self:UpdateTime()
                        self:UpdateClockStyle()
                    end,
                    default = self.db.showInGameTime,
                },
                {
                    type = "colorpicker",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_COLOR"),
                    getFunc = function()
                        return self:ConvHexToRGB(self.db.inGameTimeColor)
                    end,
                    setFunc = function(r, g, b)
                        self.db.inGameTimeColor = self:ConvRGBToHex(r, g, b)
                        self:UpdateClockStyle()
                    end,
                    disabled = function()
                        return not self.db.showInGameTime
                    end
                },
                {
                    type = "slider",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CLOCK_INGAMETIME_FONTRATIO"),
                    min = 0,
                    max = 20,
                    getFunc = function()
                        return self.db.inGameTimeDelta
                    end,
                    setFunc = function(value)
                        self.db.inGameTimeDelta = value
                        self:UpdateClockStyle()
                    end,
                    disabled = function()
                        return not self.db.showInGameTime
                    end
                },
            },
        },
        {
            type = "submenu",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CHAT_TITLE"),
            tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_CHAT"),
            width = "full",
            controls = {
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CHAT_SHOW_TIMESTAMP"),
                    getFunc = function()
                        return self.db.showChatTimestamp
                    end,
                    setFunc = function(value)
                        self.db.showChatTimestamp = value
                    end,
                    default = self.db.showChatTimestamp,
                },
                {
                    type = "dropdown",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CHAT_SHOW_RANKICONS"),
                    choices = showRankIconLabels,
                    choicesValues = showRankIcon,
                    getFunc = function()
                        return self.db.showChatGuildIcon
                    end,
                    setFunc = function(value)
                        self.db.showChatGuildIcon = value
                    end,
                    default = self.db.showChatGuildIcon,
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_CHAT_SHOW_RANKICONSCOLOR"),
                    getFunc = function()
                        return self.db.showChatGuildIconColor
                    end,
                    setFunc = function(value)
                        self.db.showChatGuildIconColor = value
                    end,
                    default = self.db.showChatGuildIconColor,
                },
            },
        },
        {
            type = "submenu",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ANNOUNCEMENTS_TITLE"),
            tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_ANNOUNCEMENTS"),
            width = "full",
            controls = {
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ANNOUNCEMENTS_SHOW_SCREEN"),
                    getFunc = function()
                        return self.db.showAnnouncements
                    end,
                    setFunc = function(value)
                        self.db.showAnnouncements = value
                    end,
                    default = self.db.showAnnouncements,
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_ANNOUNCEMENTS_SHOW_CHAT"),
                    getFunc = function()
                        return self.db.showChatAnnouncements
                    end,
                    setFunc = function(value)
                        self.db.showChatAnnouncements = value
                    end,
                    default = self.db.showChatAnnouncements,
                },
            },
        },
        {
            type = "submenu",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_TITLE"),
            tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_REPAIR"),
            width = "full",
            controls = {
                {
                    type = "header",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_STORES_HEADER"),
                },
                {
                    type = "dropdown",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_STORES"),
                    tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_REPAIR_STORES"),
                    choices = repairOptionLookup,
                    choicesValues = repairOption,
                    getFunc = function() return self.db.storeRepairMode end,
                    setFunc = function(value)   self.db.storeRepairMode = value end,
                    default = self.db.storeRepairMode,
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_PRINT"),
                    getFunc = function() return self.db.verboseShops end,
                    setFunc = function(value)   self.db.verboseShops = value end,
                    default = self.db.verboseShops,
                },
                {
                    type = "header",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_HEADER"),
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_KIT"),
                    getFunc = function() return self.db.useAnyKit end,
                    setFunc = function(value)   self.db.useAnyKit = value end,
                    default = self.db.useAnyKit,
                },
                {
                    type = "slider",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_THRESHOLD"),
                    tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_REPAIR_THRESHOLD"),
                    min = 0, max = 100, step = 1,
                    getFunc = function() return self.db.repairThreshold end,
                    setFunc = function(value)   self.db.repairThreshold = value end,
                    default = self.db.repairThreshold,
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_PRINT"),
                    getFunc = function() return self.db.verboseKits end,
                    setFunc = function(value)   self.db.verboseKits = value end,
                    default = self.db.verboseKits,
                },
                {
                    type = "header",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_RECHARGE_HEADER"),
                },
                {
                    type = "dropdown",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_RECHARGE_AUTO"),
                    choices = rechargeOptionLookup,
                    choicesValues = rechargeOption,
                    getFunc = function() return self.db.rechargeMode end,
                    setFunc = function(value)   self.db.rechargeMode = value end,
                    default = self.db.rechargeMode,
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_RECHARGE_GEM"),
                    getFunc = function() return self.db.useAnyGem end,
                    setFunc = function(value)   self.db.useAnyGem = value end,
                    default = self.db.useAnyGem,
                },
                {
                    type = "slider",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_RECHARGE_THRESHOLD"),
                    tooltip = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_TOOLTIP_RECHARGE_THRESHOLD"),
                    min = 0, max = 100, step = 1,
                    getFunc = function() return self.db.rechargeThreshold end,
                    setFunc = function(value)   self.db.rechargeThreshold = value end,
                    default = self.db.rechargeThreshold,
                },
                {
                    type = "checkbox",
                    name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_REPAIR_PRINT"),
                    getFunc = function() return self.db.verboseGems end,
                    setFunc = function(value)   self.db.verboseGems = value end,
                    default = self.db.verboseGems,
                },
            }
        },
        {
            type = "button",
            name = ArcanumGuildHall.GetDefaultLocaleString("SETTINGS_SHOW_CHANGELOGBUTTON"),
            func = function()
                ArcanumGuildHall_Changelog:SetHidden(false)
            end,
            width = "half",
        },
    }
    self.panel = LAM2:RegisterAddonPanel(self.name .. "Options", panelData)
    LAM2:RegisterOptionControls(self.name .. "Options", optionsTable)

    local function panelShown(currentPanel)
        if currentPanel == self.panel then
            panelOpen = true
            ArcanumGuildHall:UpdateAnchors(panelOpen)
        end
    end

    local function panelHidden()
        panelOpen = false
        ArcanumGuildHall:UpdateAnchors(panelOpen)
        ArcanumClock:SetHidden(true)
    end

    CALLBACK_MANAGER:RegisterCallback("LAM-PanelOpened", panelShown)
    CALLBACK_MANAGER:RegisterCallback("LAM-PanelClosed", panelHidden)
end
