local ArcanumGuildHall = _G['ArcanumGuildHall']
local EVENT_MANAGER = EVENT_MANAGER

local res = ArcanumGuildHallMediaRes
local calculateConstraints = SharedChatContainer.CalculateConstraints

-- Convert RGBToHex and HexToRGB
function ArcanumGuildHall:ConvRGBToHex(r, g, b)
    return string.format("|c%.2x%.2x%.2x", zo_floor(r * 255), zo_floor(g * 255), zo_floor(b * 255))
end

function ArcanumGuildHall:ConvHexToRGB(colourString)
    local r = tonumber(string.sub(colourString, 3, 4), 16) or 255
    local g = tonumber(string.sub(colourString, 5, 6), 16) or 255
    local b = tonumber(string.sub(colourString, 7, 8), 16) or 255
    return r / 255, g / 255, b / 255
end

function ArcanumGuildHall.Debounce(namespace, timeout, callbackFunc)
    EVENT_MANAGER:UnregisterForUpdate(namespace)

    EVENT_MANAGER:RegisterForUpdate(namespace, timeout, function()
        EVENT_MANAGER:UnregisterForUpdate(namespace)
        callbackFunc()
    end)
end

-- Maximal Characters for MotD and Guild Description
local function createLabel(name, anchor, text, dimension, offset, hidden, pos)

    local guiMaxChars = WINDOW_MANAGER:CreateControl(name, anchor, CT_LABEL)

    guiMaxChars:SetFont("ZoFontGame")
    guiMaxChars:SetDimensions(dimension[1], dimension[2])
    guiMaxChars:SetAnchor(LEFT, anchor, pos, offset[1], offset[2])
    guiMaxChars:SetText(text)
    guiMaxChars:SetVerticalAlignment(LEFT)
    guiMaxChars:SetHidden(hidden)

    return guiMaxChars
end

local DescriptionLeft = createLabel("DescriptionLeftLabel", ZO_GuildHomeInfoDescriptionSavingEdit, "100/2048", { 100, 30 }, { -70, -51 }, false, TOPRIGHT)
local MotDLeft = createLabel("MotDLeftLabel", ZO_GuildHomeInfoMotDSavingEdit, "100/2048", { 100, 30 }, { -70, -51 }, false, TOPRIGHT)

local MotDHandler = ZO_GuildHomeInfoMotDSavingEdit:GetHandler("OnTextChanged")
local DescriptionHandler = ZO_GuildHomeInfoDescriptionSavingEdit:GetHandler("OnTextChanged")

ZO_GuildHomeInfoMotDSavingEdit:SetHandler("OnTextChanged", function()
    MotDHandler()
    local guiMaxChars = ZO_GuildHomeInfoMotDSavingEdit
    local length = string.len(guiMaxChars:GetText())
    local maxTextMotD = res.Ccolor7 .. "/" .. "2048"
    local color = res.Ccolor7

    if length > 2000 then
        color = res.Ccolor9
    end
    MotDLeft:SetText(color .. length .. maxTextMotD)
end)

ZO_GuildHomeInfoDescriptionSavingEdit:SetHandler("OnTextChanged", function()
    DescriptionHandler()
    local guiMaxChars = ZO_GuildHomeInfoDescriptionSavingEdit
    local length = string.len(guiMaxChars:GetText())
    local maxTextDescription = res.Ccolor7 .. "/" .. "2048"
    local color = res.Ccolor7

    if length > 2000 then
        color = res.Ccolor9
    end
    DescriptionLeft:SetText(color .. length .. maxTextDescription)
end)

-- Set max Chat Window size
function ArcanumGuildHall:SetChatHook()
    function SharedChatContainer.CalculateConstraints(...)
        local self = ...
        local w, h = GuiRoot:GetDimensions()
        self.system.maxContainerWidth, self.system.maxContainerHeight = w * 0.95, h * 0.95
        return calculateConstraints(...)
    end
end

-- Chat or Screen Announcements
local alliance = GetUnitAlliance("player")
local sound

if alliance == 1 then
    sound = SOUNDS.EMPEROR_CORONATED_ALDMERI
end

if alliance == 2 then
    sound = SOUNDS.EMPEROR_CORONATED_EBONHEART
end

if alliance == 3 then
    sound = SOUNDS.EMPEROR_CORONATED_DAGGERFALL
end

function ArcanumGuildHall.screenAnnouncement(msgText)

    local msgParams = CENTER_SCREEN_ANNOUNCE:CreateMessageParams(CSA_CATEGORY_LARGE_TEXT, sound)
    msgParams:SetText(msgText)
    msgParams:SetCSAType(CENTER_SCREEN_ANNOUNCE_TYPE_COUNTDOWN)

    CENTER_SCREEN_ANNOUNCE:AddMessageWithParams(msgParams)

end

local ANNOUNCEMENT_PREFIX = "AAI:"
local PLAYER_NAME = "@Niwasaka"
local ANNOUNCEMENT_DELAY = 2000
local MAX_LINE_LENGTH = 60

function ArcanumGuildHall.OnGuildDescChanged(event, guildId, fromDisplay)

    if not ArcanumGuildHall.db.showAnnouncements and not ArcanumGuildHall.db.showChatAnnouncements then
        return
    end

    local memberIndex = GetGuildMemberIndexFromDisplayName(ArcanumGuildHall.guildId, PLAYER_NAME)
    local _, memberNote, rankIndex = GetGuildMemberInfo(ArcanumGuildHall.guildId, memberIndex)

    local icon = GetGuildName(guildId)
    --local desc = GetGuildDescription(guildId)
    if nil == icon or nil == memberNote then
        return
    end

    local alertName = res.Ccolor3 .. ArcanumGuildHall.GetDefaultLocaleString("ANNOUNCEMENTS_TEXT") .. " "
    local alertIconSm = res.IconAA .. " "
    local alertIconBig = res.IconAAB .. " "
    local msgs = { }
    local indexEnd
    local _, indexStart = string.find(string.lower(memberNote), string.lower(ANNOUNCEMENT_PREFIX), 1, true)

    while nil ~= indexStart do

        indexStart = indexStart + 1
        indexEnd = string.find(memberNote, "\n", indexStart, true)

        local msg = string.sub(memberNote, indexStart, indexEnd and indexEnd - 1 or nil)
        while nil ~= msg and "" ~= msg and "\n" ~= msg do

            table.insert(msgs, string.sub(msg, 1, MAX_LINE_LENGTH))
            msg = string.sub(msg, MAX_LINE_LENGTH + 1)
        end
        indexStart = indexEnd
    end

    if 0 < #msgs then

        if ArcanumGuildHall.db.showChatAnnouncements then
            local timeStamp = ArcanumGuildHall:getTime()
            CHAT_ROUTER:AddSystemMessage(alertIconSm .. alertName .. timeStamp)
        end

        if ArcanumGuildHall.db.showAnnouncements then
            ArcanumGuildHall.screenAnnouncement(alertIconBig .. alertName)
        end

        local delay = ANNOUNCEMENT_DELAY

        for _, m in ipairs(msgs) do

            if ArcanumGuildHall.db.showChatAnnouncements then
                CHAT_ROUTER:AddSystemMessage(res.Ccolor7 .. m)
            end

            if ArcanumGuildHall.db.showAnnouncements then
                zo_callLater(function()
                    ArcanumGuildHall.screenAnnouncement(m)
                end, delay)
                delay = delay + ANNOUNCEMENT_DELAY
            end
        end
    end
end

-- Take one item instead of all
local LCM = LibCustomMenu

function ArcanumGuildHall:TakeOneFromInventory(inventorySlot, _itemId)
    local slotType = ZO_InventorySlot_GetType(inventorySlot)
    local bagId, slotIndex = ZO_Inventory_GetBagAndIndex(inventorySlot)

    if not slotIndex then
        return
    end

    local itemLink = GetItemLink(bagId, slotIndex)
    local itemId = GetItemLinkItemId(itemLink)

    if _itemId ~= itemId then
        return
    end

    local targetSlot = FindFirstEmptySlotInBag(BAG_BACKPACK)

    if not targetSlot then
        CHAT_ROUTER:AddSystemMessage(res.Ccolor9 .. ArcanumGuildHall.GetDefaultLocaleString("TAKE_FAILURE"))
        return
    end

    local quantity = GetSlotStackSize(bagId, slotIndex)
    CHAT_ROUTER:AddSystemMessage(res.Ccolor2 .. zo_strformat(ArcanumGuildHall.GetDefaultLocaleString("TAKE_PROCESS"), quantity, itemLink))

    if slotType == SLOT_TYPE_BANK_ITEM then
        CallSecureProtected("RequestMoveItem", bagId, slotIndex, BAG_BACKPACK, targetSlot, 1)
    elseif slotType == SLOT_TYPE_GUILD_BANK_ITEM then
        EVENT_MANAGER:RegisterForEvent(self.addonName, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, self:SplitItems(itemId, quantity))
        EVENT_MANAGER:AddFilterForEvent(self.addonName, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, REGISTER_FILTER_BAG_ID, BAG_BACKPACK)
        TransferFromGuildBank(slotIndex)
    end
end

function ArcanumGuildHall:SplitItems(itemId, quantity)
    return function(_, bagId, slotIndex, _, _, _, _)
        if bagId ~= BAG_BACKPACK then
            return
        end

        local itemLink = GetItemLink(bagId, slotIndex)
        local _itemId = GetItemLinkItemId(itemLink)
        if _itemId ~= itemId then
            return
        end

        local _quantity = GetSlotStackSize(bagId, slotIndex)
        if quantity ~= _quantity then
            return
        end

        EVENT_MANAGER:UnregisterForEvent(self.addonName, EVENT_INVENTORY_SINGLE_SLOT_UPDATE)

        local targetSlot = FindFirstEmptySlotInBag(BAG_BACKPACK)

        EVENT_MANAGER:RegisterForEvent(self.addonName, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, self:ReturnItems(bagId, slotIndex, targetSlot))
        EVENT_MANAGER:AddFilterForEvent(self.addonName, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, REGISTER_FILTER_BAG_ID, BAG_BACKPACK)

        CallSecureProtected("RequestMoveItem", bagId, slotIndex, BAG_BACKPACK, targetSlot, 1)
        CHAT_ROUTER:AddSystemMessage(res.IconCheck .. res.Ccolor2 .. (ArcanumGuildHall.GetDefaultLocaleString("TAKE_RETURN")))
    end
end

function ArcanumGuildHall:ReturnItems(bagId, slotIndex, targetSlot)
    return function(_, _, _slotIndex, _, _, _, _)
        if targetSlot ~= _slotIndex then
            return
        end
        EVENT_MANAGER:UnregisterForEvent(self.addonName, EVENT_INVENTORY_SINGLE_SLOT_UPDATE)
        TransferToGuildBank(bagId, slotIndex)
    end
end

function ArcanumGuildHall:ShowInventoryContextMenu()
    LCM:RegisterContextMenu(function(inventorySlot, slotActions)
        local slotType = ZO_InventorySlot_GetType(inventorySlot)
        local bagId, slotIndex = ZO_Inventory_GetBagAndIndex(inventorySlot)

        if slotType ~= SLOT_TYPE_BANK_ITEM and slotType ~= SLOT_TYPE_GUILD_BANK_ITEM then
            return
        end

        if slotType == SLOT_TYPE_GUILD_BANK_ITEM then
            local guildId = GetSelectedGuildBankId()

            if not guildId or not DoesGuildHavePrivilege(guildId, GUILD_PRIVILEGE_BANK_DEPOSIT) or
                    not (DoesPlayerHaveGuildPermission(guildId, GUILD_PERMISSION_BANK_DEPOSIT) and DoesPlayerHaveGuildPermission(guildId, GUILD_PERMISSION_BANK_WITHDRAW)) then
                return
            end
        end

        if (slotType == SLOT_TYPE_BANK_ITEM and not CheckInventorySpaceSilently(1)) or
                (slotType == SLOT_TYPE_GUILD_BANK_ITEM and not CheckInventorySpaceSilently(2)) or
                GetSlotStackSize(bagId, slotIndex) <= 1 then
            return
        end

        local itemLink = GetItemLink(bagId, slotIndex)
        local itemId = GetItemLinkItemId(itemLink)

        AddCustomMenuItem(res.Ccolor8 .. ArcanumGuildHall.GetDefaultLocaleString("TAKE_LABEL"), function()
            self:TakeOneFromInventory(inventorySlot, itemId)
        end)
    end, LCM.CATEGORY_LATE)
end